/***************************************************************************
 *   basecode - archive of useful C++ classes                              *
 *   Copyright (C) 2004 by Michal Turek - Woq                              *
 *   WOQ (at) seznam.cz, http://woq.nipax.cz/                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/*
 * Author of this class: Radomir Vrana <rvalien (at) c-box.cz>
 * A lot of changes: Michal Turek - Woq <WOQ (at) seznam.cz>
 */

#ifndef __CIMAGE_H__
#define __CIMAGE_H__

#include <string>
#include <iostream>

#include <SDL.h>
#include <SDL_opengl.h>
#include <SDL_image.h>// Don't forget to link library file!

#include "basecode.h"

using namespace std;

namespace basecode
{

class CImage
{
public:
	CImage();
	CImage(const string & filename);
			//throw(CFileNotFound, CBadFileFormat, std::bad_alloc);
	~CImage();

	void Destroy();
	void Init(const string & filename);
			//throw(CFileNotFound, CBadFileFormat, std::bad_alloc);

	Uint16 GetWidth() const { return m_width; }
	Uint16 GetHeight() const { return m_height; }
	Uint16 GetBytesPerPixel() const { return m_bytes_per_pixel; }
	Uint8* GetDataPtr() const { return m_pdata; }

	int GetSize() const { return m_width * m_height * m_bytes_per_pixel; }
	bool IsEmpty() const { return (m_pdata == NULL); }

	// Return 0 if no image or if not RGB(A)
	GLuint CreateGLTexture(bool mipmaps = true, bool border = false) const;

private:
	Uint8 *m_pdata;
	Uint16 m_width;
	Uint16 m_height;
	Uint16 m_bytes_per_pixel;
};

}

#endif
